/** @jsx jsx */
import { Children, forwardRef, Fragment, ReactNode } from 'react';

import { css, jsx } from '@emotion/react';

import { token } from '@atlaskit/tokens';

import Text from './text.partial';
import type { BasePrimitiveProps, NonTextChildren } from './types';

interface InlineProps extends BasePrimitiveProps {
  /**
   * Used to align children along the cross axis.
   */
  alignItems?: FlexAlignItems;
  /**
   * Used to align children along the main axis.
   */
  justifyContent?: FlexJustifyContent;
  /**
   * Sets whether children are forced onto one line or can wrap onto multiple lines
   */
  flexWrap?: FlexWrap;
  /**
   * Token representing gap between children.
   */
  gap: ColumnGap;
  /**
   * Renders a divider between children.
   * If a string is provided it will automatically be wrapped in a `<Text>` component.
   */
  divider?: ReactNode;
  /**
   * Elements to be rendered inside the Inline.
   */
  children: NonTextChildren;
}

type FlexAlignItems = keyof typeof flexAlignItemsMap;
const flexAlignItemsMap = {
  center: css({ alignItems: 'center' }),
  baseline: css({ alignItems: 'baseline' }),
  flexStart: css({ alignItems: 'flex-start' }),
  flexEnd: css({ alignItems: 'flex-end' }),
  start: css({ alignItems: 'start' }),
  end: css({ alignItems: 'end' }),
};

type FlexJustifyContent = keyof typeof flexJustifyContentMap;
const flexJustifyContentMap = {
  center: css({ justifyContent: 'center' }),
  flexStart: css({ justifyContent: 'flex-start' }),
  flexEnd: css({ justifyContent: 'flex-end' }),
  start: css({ justifyContent: 'start' }),
  end: css({ justifyContent: 'end' }),
};

type FlexWrap = keyof typeof flexWrapMap;
const flexWrapMap = {
  wrap: css({ flexWrap: 'wrap' }),
};

const baseStyles = css({
  display: 'flex',
  boxSizing: 'border-box',
  flexDirection: 'row',
});

/**
 * __Inline__
 *
 * Inline is a primitive component based on flexbox that manages the horizontal layout of direct children.
 * Renders a `div` by default.
 *
 */
const Inline = forwardRef<HTMLDivElement, InlineProps>(
  (
    {
      gap,
      alignItems,
      justifyContent,
      flexWrap,
      divider,
      UNSAFE_style,
      children,
      testId,
    },
    ref,
  ) => {
    const dividerComponent =
      typeof divider === 'string' ? <Text>{divider}</Text> : divider;

    return (
      <div
        style={UNSAFE_style}
        css={[
          baseStyles,
          gap && columnGapMap[gap],
          alignItems && flexAlignItemsMap[alignItems],
          justifyContent && flexJustifyContentMap[justifyContent],
          flexWrap && flexWrapMap[flexWrap],
        ]}
        data-testid={testId}
        ref={ref}
      >
        {Children.map(children, (child, index) => {
          return (
            <Fragment>
              {divider && index > 0 ? dividerComponent : null}
              {child}
            </Fragment>
          );
        })}
      </div>
    );
  },
);

Inline.displayName = 'Inline';

export default Inline;

/**
 * THIS SECTION WAS CREATED VIA CODEGEN DO NOT MODIFY {@see http://go/af-codegen}
 * @codegen <<SignedSource::a3c9f0528d75060e7cf9617878520e18>>
 * @codegenId spacing
 * @codegenCommand yarn codegen-styles
 * @codegenParams ["columnGap"]
 * @codegenDependency ../../../tokens/src/artifacts/tokens-raw/atlassian-light.tsx <<SignedSource::07ef29d58a2b23af8b098515466d7e22>>
 */
const columnGapMap = {
  'scale.0': css({
    columnGap: token('spacing.scale.0', '0'),
  }),
  'scale.025': css({
    columnGap: token('spacing.scale.025', '0.125rem'),
  }),
  'scale.050': css({
    columnGap: token('spacing.scale.050', '0.25rem'),
  }),
  'scale.075': css({
    columnGap: token('spacing.scale.075', '0.375rem'),
  }),
  'scale.100': css({
    columnGap: token('spacing.scale.100', '0.5rem'),
  }),
  'scale.150': css({
    columnGap: token('spacing.scale.150', '0.75rem'),
  }),
  'scale.200': css({
    columnGap: token('spacing.scale.200', '1rem'),
  }),
  'scale.250': css({
    columnGap: token('spacing.scale.250', '1.25rem'),
  }),
  'scale.300': css({
    columnGap: token('spacing.scale.300', '1.5rem'),
  }),
  'scale.400': css({
    columnGap: token('spacing.scale.400', '2rem'),
  }),
  'scale.500': css({
    columnGap: token('spacing.scale.500', '2.5rem'),
  }),
  'scale.600': css({
    columnGap: token('spacing.scale.600', '3rem'),
  }),
};

export type ColumnGap = keyof typeof columnGapMap;

/**
 * @codegenEnd
 */
